# -*- coding: utf-8 -*-
import datetime

from django.contrib import admin
from django.contrib.admin.widgets import FilteredSelectMultiple
from django.utils.translation import ugettext_lazy as _
from django.utils.translation import ugettext as ugt
from django.utils.formats import date_format
from django.conf.urls.defaults import *
from django import forms
from django.conf import settings
from django.db.models import Q
from django.forms.models import ModelChoiceField

from jumbo.admin.options import ExtendibleModelAdmin
from jumbo.apps.autocomplete import autocomplete
from jumbo.apps.autocomplete import AutocompleteModelChoiceField
from jumbo.apps.project.models import Project
from jumbo.forms.models import AdminModelForm
from jumbo.admin.options import OrderedModelAdmin, ExtendibleModelAdmin, SettingsAdmin
from jumbo.core.urlresolvers import reverse
from jumbo.forms import DateWidget

from document.models import DocumentProject, Document, Category, OtherCategory, Default, DocumentUser, Link, Tipology, Settings
from organization.models import Organization, Contact

### INLINE
#class PermInline(admin.TabularInline):
    #model = Perm
    #form = PermAdminForm
    #extra = 1
    #fields = ('group', 'list_perms', 'add_perms', 'edit_perms')
    #radio_fields = {"list_perms": admin.HORIZONTAL, "add_perms": admin.HORIZONTAL, "edit_perms": admin.HORIZONTAL}


# SEARCH
class DocumentSearchForm(forms.ModelForm):
    class Meta:
        model = Document
        fields = ('status', 'projects', 'category', 'tipology', 'support', 'assigned_by', 'assigned_to', 'keywords')
        display_fields = (
            'status', 'projects', 'keywords',
            ('title__icontains', 'organization__code__icontains', 'organization__name__icontains'),
            ('person__last_name__icontains', 'person__first_name__icontains'),
            ('category', 'tipology',),
            'support',
            ('assigned_by', 'assigned_to'),
            ('date_doc__year', 'search_in_all_years'),
            ('date_doc__gte', 'date_doc__lte'),
            ('number__gte', 'number__lte'),
            ('protocol_number__gte', 'protocol_number__lte'),
        )


    def __init__(self, *args, **kws):
        super(DocumentSearchForm, self).__init__(*args, **kws)
        #self.fields['projects__in'] = forms.CharField(la)
        self.fields['title__icontains'] = forms.CharField(label=_("Title"), required=False)
        self.fields['organization__code__icontains'] = forms.CharField(label=_("Organization Code"), required=False)
        self.fields['organization__name__icontains'] = forms.CharField(label=_("Organization Name"), required=False)
        self.fields['person__last_name__icontains'] = forms.CharField(label=_("Last Name"), required=False)
        self.fields['person__first_name__icontains'] = forms.CharField(label=_("First Name"), required=False)
        self.fields['date_doc__gte'] = forms.DateField(label=_("Date from"), widget= DateWidget, required=False)
        self.fields['date_doc__lte'] = forms.DateField(label=_("Date to"), widget= DateWidget, required=False)
        self.fields['number__gte'] = forms.CharField(label=_("Document Number from"), required=False)
        self.fields['number__lte'] = forms.CharField(label=_("Document Number to"), required=False)
        self.fields['protocol_number__gte'] = forms.IntegerField(label=_("Protocol Number from"), required=False)
        self.fields['protocol_number__lte'] = forms.IntegerField(label=_("Protocol Number to"), required=False)
        self.fields['search_in_all_years'] = forms.BooleanField(label=_("Search In All Years"), required=False)
        self.fields['date_doc__year'] = forms.CharField(label=_("Year"), required=False)

    # Modifico il filtro di ricerca in modo da filtrare sempre in base all'anno corrente, tranne nel caso in cui
    # nei parametri di ricerca venga specificato un anno oppure venga flaggata la checkbox search_in_all_years
    def change_search_filter(self, lookup_params, qs):
        if not lookup_params.has_key('date_doc__year') and not lookup_params.has_key('search_in_all_years'):
            qs = qs.filter(date_doc__year = datetime.date.today().year)

        if lookup_params.has_key('date_doc__lte'):
            date_doc = "%s 23:59:59" % lookup_params['date_doc__lte']
            del lookup_params['date_doc__lte']
            qs = qs.filter(date_doc__lte=date_doc)

        if lookup_params.has_key('search_in_all_years'):
            del lookup_params['search_in_all_years']
        return lookup_params, qs

## GENERAL
#########################################
# DOCUMENT                              #
#########################################
class DownloadableByMultipleChoiceField(ModelChoiceField):
    def label_from_instance(self, obj):
        try:
            return "%s --- %s" % (obj.user.contact.organization, obj.user.contact)
        except Exception, e:
            return "%s" % obj.user

class DocumentAdminForm(AdminModelForm):
    class Meta:
        model = Document
    class Media:
        js = ('document/js/document_form.js',)

    def __init__(self, *args, **kws):
        super(DocumentAdminForm, self).__init__(*args, **kws)
        self.fields['organization'] = AutocompleteModelChoiceField(
            'document_organization', label=_("Organization"),
            on_select_token='on_select_org',
            on_delete_token='on_delete_org',
            required=False,
        )
        contacts = Contact.objects.none()
        organization = self.get_object_id('organization')

        if organization:
            contacts = Contact.objects.filter(organization__id=organization)

        self.fields['person'].queryset = contacts
        self.fields['person'].required = True
#        self.fields['downloadable_by'] = DownloadableByMultipleChoiceField(
#            queryset=DocumentUser.objects.all(), required=False,
#            widget=FilteredSelectMultiple(
#                verbose_name=_('Downloadable by'),
#                is_stacked=False
#            )
#        )

class DocumentUserAdmin(ExtendibleModelAdmin):
    fields = ('user', 'enabled')
    list_per_page = 30
    list_display = ('user', 'enabled')


class DocumentAdmin(ExtendibleModelAdmin):
    class Media:
        css = {"all": ("document/css/admin_document.css",)}

    form = DocumentAdminForm
    search_form = DocumentSearchForm
    list_per_page = 30

    fieldsets = (
        (_('General Information'), {'fields': (
                'is_public', 'status', 'title', ('support', 'category',), 'file',
                'reception_date', 'date_doc', 'number', 'protocol_number', 'direction', 'projects',
                ('organization', 'person',), ('assigned_by', 'assigned_to',),
                'notify_group', 'downloadable_by', 'keywords', 'pages', 'note'
        )}),
    )

    filter_horizontal = ('projects', 'downloadable_by', 'notify_group')
    list_display = (
        'get_short_status', 'get_creator', 'get_is_public', 'get_working', 'get_detail', 'category',
        'get_doc_number', 'get_prot_number', 'get_year', 'support', 'title', 'get_groups', 'pages', 'get_linked_documents',
        'get_edit_icon', 'get_link_file', 'get_delete_icon'
    )
    list_display_csv   = ('get_short_status', 'title', 'organization', 'person', 'file', 'number')
    list_display_links = ['title']
    detail_display     = (('title', ('date_create','date_last_modify',),))
    default_group_by   = 'category'
    ordering           = ('-status',)
    save_as            = True
    save_on_top        = True
    actions            = ['join', 'split']
    date_hierarchy     = 'date_doc'


    def get_urls(self):
        from django.conf.urls.defaults import patterns, url
        urls = super(DocumentAdmin, self).get_urls()
        my_urls = patterns('',
            url(r'^(?P<object_id>\d+)/file_with_label/$'   , self.admin_site.admin_view(self.download_file_with_label)
                                                           , name='document_document_file_with_label'),
        )
        return my_urls + urls


    def download_file_with_label(self, request, object_id):
        # Creo al volo il file con il watermark (o label) e lo ritorno in un HttpResponse
        #
        import StringIO
        from pyPdf import PdfFileReader, PdfFileWriter
        from django.http import HttpResponse, Http404

        obj = Document.objects.get(id=object_id)

        # Apro il file dove deve essere appiccicato il watermark
        orig_file = PdfFileReader(obj.file)
        page1 = orig_file.getPage(0)
        watermark = PdfFileReader(obj.etichetta)

        # Appiccico il watermark nella prima pagina
        page1.mergePage(watermark.getPage(0))

        # finally, write "output" to document-output.pdf
        output_packet = StringIO.StringIO()
        output = PdfFileWriter()
        output.addPage(page1)
        for i in range(1, orig_file.getNumPages()):
            page = orig_file.getPage(i)
            output.addPage(page)
        #outputStream = file("/tmp/watermark1.pdf", "wb")
        output.write(output_packet)
        output_packet.seek(0)

        response = HttpResponse(output_packet.read())

        response['Content-Type'] = 'application/pdf'
        return response


    def create_label(self, request, queryset):
        # Action che crea un pdf solo con la label che poi verra' appiccicata
        # al file solo nel momento in cui viene richiesto il documento con la label
        from django.core.files.base import ContentFile
        from django.template import Context
        from jumbo.utils.ooutils import OOUtils

        o = OOUtils()
        label_created = 0

        for d in queryset:
            filename = d.file.name.split('/')[-1]
            raw_pdf = o.save_pdf(request, 'admin/document/document/watermark.odt', obj=d, get_raw_string=True)
            file_content = ContentFile(raw_pdf.read())
            d.etichetta.save(filename, file_content)
            label_created += 1

        if label_created == 1:
            message_bit = _("1 Document Label Created")
        else:
            message_bit = _("%(row)s Documents Label Created") % {'row':label_created}
        self.message_user(request, _("%(msg)s") % {'msg': message_bit})
    create_label.short_description = _("Create Label")


    def save_model(self, request, obj, form, change):
        super(DocumentAdmin, self).save_model(request, obj, form, change)

        # Se e' stata selezionata una categoria monitorata via mail
        # spedisco la mail ai relativi destinatari
        if form.cleaned_data.has_key('other_category'):
            for cat in form.cleaned_data['other_category']:
                if cat.notify_to_add_document:
                    obj.send_mail_add_document_to_category(cat, change)


    def set_as_completed(self, request, queryset):
        rows_updated = queryset.update(status='CM')
        if rows_updated == 1:
            message_bit = ugt("1 object was")
        else:
            message_bit = ugt("%(row)s objects were") % {'row': rows_updated}
        self.message_user(
            request, ugt("%(msg)s successfully setted.") % {'msg': message_bit}
        )
    set_as_completed.short_description = _("Set %(verbose_name_plural)s as completed")
    admin.site.add_action(set_as_completed)

    def set_as_to_complete(self, request, queryset):
        rows_updated = queryset.update(status='TC')
        if rows_updated == 1:
            message_bit = ugt("1 object was")
        else:
            message_bit = ugt("%(row)s objects were") % {'row': rows_updated}
        self.message_user(
            request, ugt("%(msg)s successfully setted.") % {'msg': message_bit}
        )
    set_as_to_complete.short_description = _("Set %(verbose_name_plural)s as to complete")
    admin.site.add_action(set_as_to_complete)

    def get_actions(self, request):
        actions = super(DocumentAdmin, self).get_actions(request)
        del actions['set_active']
        del actions['set_disactive']
        return actions

    def get_creator(self, obj):
        return obj.creator
    get_creator.short_description = _("Inserted By")

    def join(self, request, queryset):
        link=Link()
        rows_linked = 0
        for d in queryset:
            link.document=d
            link.save()
            link.id=None
            rows_linked += 1
        if rows_linked == 1:
            message_bit = _("1 Document Linked")
        else:
            message_bit = _("%(row)s Documents Linked") % {'row':rows_linked}
        self.message_user(request, _("%(msg)s") % {'msg': message_bit})
    join.short_description = _("Link Documents")

    def split(self, request, queryset):
        rows_splitted = 0
        for d in queryset:
            Link.objects.filter(document=d).delete()
            rows_splitted += 1
        if rows_splitted == 1:
            message_bit = _("1 Document Splitted")
        else:
            message_bit = _("%(row)s Documents Splitted") % {'row':rows_splitted}
        self.message_user(request, _("%(msg)s") % {'msg': message_bit})
    split.short_description = _("Split Documents")

    def get_year(self, obj):
        if obj.date_doc:
            return obj.date_doc.strftime("%Y")
        return ""
    get_year.short_description = _("Year")

    def get_linked_documents(self, obj):
        links = Link.objects.filter(document=obj)
        nlink_list = []
        for link in links:
            nlink_list.append(link.number)
        other_link_list = Link.objects.filter(number__in=nlink_list).exclude(document=obj)
        docs = ""
        for link in other_link_list:
            docs += "%s," % link.document.id
        if docs:
            return """
            <a
                href='/admin/document/document/?id__in=%(list_link)s%(id_obj)s&search_in_all_years=on'
                title='List linked documents'
            >
                <img src="%(url)sadmin/images/icon_addlink.gif" />
            </a>
            """ % {
                'url': settings.MEDIA_URL,
                'list_link': docs,
                'id_obj': obj.id
            }
        return ""
    get_linked_documents.short_description = _("L")
    get_linked_documents.allow_tags = True

    #def get_date_doc(self, obj):
    #    if obj.date_doc:
    #        return obj.date_doc.strftime("%d/%m/%Y")
    #    return ""
    #get_date_doc.short_description = _("Date Document")
    #get_date_doc.admin_order_field = 'date_doc'


    def get_short_status(self, obj):
        return obj.status
    get_short_status.short_description = _("ST")
    get_short_status.admin_order_field = 'status'


    def get_is_public(self, obj):
        return obj.is_public
    get_is_public.short_description = _("P")
    get_is_public.admin_order_field = 'is_public'
    get_is_public.boolean = True


    def get_working(self, obj):
        return obj.working
    get_working.short_description = _("L")
    get_working.admin_order_field = 'working'
    get_working.boolean = True


    def get_doc_number(self, obj):
        return obj.number
    get_doc_number.short_description = ("Doc N.")
    get_doc_number.admin_order_field = 'number'


    def get_prot_number(self, obj):
        return obj.protocol_number
    get_prot_number.short_description = ("Prot N.")
    get_prot_number.admin_order_field = 'protocol_number'


    def get_groups(self, obj):
        g = [ x.name for x in obj.notify_group.all() ]
        return ', '.join(g)
    get_groups.short_description = _("Groups")


    def get_print_icon(self, obj):
        return """<a href="#" onclick="Dajaxice.document.print_document_label(Dajax.process, {'obj_id':%d});">P</a>""" % obj.id
    get_print_icon.short_description = _("Print")
    get_print_icon.allow_tags = True


    def get_detail(self, obj):
        organization = obj.organization.name or ""
        person = obj.person or ""
        projects = obj.get_string_projects()
        return "<b>%s</b><br/>%s<br/>%s" % (organization, person, projects)
    get_detail.short_description = _("Detail")
    get_detail.allow_tags = True
    get_detail.admin_order_field = 'organization__name'

    def get_date_doc(self, obj):
        return date_format(obj.date_doc.date())
    get_date_doc.short_description = _("Date")
    get_date_doc.allow_tags = True

    def get_link_file(self, obj):
        if obj.can_be_downloaded():
            return """<a target="_blank" href="%(url)s" title="%(title)s"><img src="%(media_url)sadmin/images/document.png" alt="%(description)s" title="%(description)s"/></a>""" % {
                'url'         : obj.file.url,
                'title'       : obj.file.name,
                'media_url'   : settings.MEDIA_URL,
                'description' : _("Download Document")
            }
        return ""
    get_link_file.short_description = _("D")
    get_link_file.allow_tags = True


    def get_link_file_with_label(self, obj):
        if obj.can_be_downloaded() and obj.etichetta:
            return """<a target="_blank" href="%(url)s" title="%(title)s"><img src="%(media_url)sadmin/images/document.png" alt="%(description)s" title="%(description)s"/></a></a>""" % {
                'url'         : reverse('admin:document_document_file_with_label', obj.id),
                'title'       : _('Download Document With Label'),
                'media_url'   : settings.MEDIA_URL,
                'description' : _("Download Document With Label")
            }
        return ""
    get_link_file_with_label.short_description = _("DL")
    get_link_file_with_label.allow_tags = True


class DocumentProjectAdminForm(AdminModelForm):
    project_ptr_id = forms.IntegerField(
        required=False,
        widget=forms.HiddenInput,
        label=""
    )
    class Meta:
        model = DocumentProject
        fields = ('managers', 'staff', 'project_ptr')

    def save(self, *args, **kwargs):
        dp = super(DocumentProjectAdminForm, self).save(commit=False)

        ## sto aggiungendo partendo da un progetto esistente
        if self.cleaned_data['project_ptr_id']:
            project_id = self.cleaned_data['project_ptr_id']
            dp.project_ptr_id = project_id
            ## Trovo il Progetto da cui si estende il DocumentProject
            project = Project.objects.get(id=project_id)
            dp.project_ptr = project
        ## sto modificando un TicketProject esistente
        else:
            project = dp.project_ptr

        dp.date_create = project.date_create
        dp.creator = project.creator
        dp.status = project.status
        dp.o_number = project.o_number
        dp.c_number = project.c_number
        dp.o_date = project.o_date
        dp.c_date = project.c_date
        dp.accepted = project.accepted
        dp.name = project.name
        dp.organization = project.organization
        dp.description = project.description
        dp.date_start = project.date_start
        dp.date_end = project.date_end
        dp.save()
        return dp

class DocumentProjectAdmin(ExtendibleModelAdmin):
    list_per_page = 30
    form = DocumentProjectAdminForm
    fieldsets = (
        (_('General Information'), {
            'fields': ('managers', 'staff', 'project_ptr_id')
        }),
    )
    list_display = ('status', 'name', 'organization', 'get_date_start', 'get_date_end',
                    'get_string_managers', 'get_string_staff')
    list_display_csv = ('status', 'name', 'organization', 'get_date_start',
                        'get_date_end', 'get_string_managers', 'get_string_staff')
    list_display_links = ['name']
    detail_display = (
        'status', 'organization', 'name', 'description',
        ('date_start', 'date_end'), ('managers', 'staff')
    )
    filter_horizontal = ('managers', 'staff')
    save_as = True
    save_on_top = True
#    actions = ['set_opened', 'set_closed']

#    def set_opened(self, request, queryset):
#        rows_updated = queryset.update(status=1)
#        if rows_updated == 1:
#            message_bit = _("1 Document Project was")
#        else:
#            message_bit = _("%(row)s Document Project were") % {'row':rows_updated}
#        self.message_user(request, _("%(msg)s successfully opened.") % {'msg': message_bit})
#    set_opened.short_description = _("Set Document Project as opened")

#    def set_closed(self, request, queryset):
#        rows_updated = queryset.update(status=0)
#        if rows_updated == 1:
#            message_bit = _("1 Document was")
#        else:
#            message_bit = _("%(row)s Document Project were") % {'row':rows_updated}
#        self.message_user(request, _("%(msg)s successfully closed.") % {'msg': message_bit})
#    set_closed.short_description = _("Set Document Project as closed")

    def get_date_start(self, obj):
        return date_format(obj.date_start)
    get_date_start.short_description = _('Date')
    get_date_start.admin_order_field = 'date_start'

    def get_date_end(self, obj):
        return date_format(obj.date_end)
    get_date_end.short_description = _('Date')
    get_date_end.admin_order_field = 'date_end'


class CategoryAdmin(ExtendibleModelAdmin):
    class Media:
        css = {"all": ("document/css/admin_document.css",)}

    #search_form = CategorySearchForm
    list_per_page = 30
    fieldsets = (
        (_('General Information'), {
            'fields': ('status', 'code', 'name', 'email', 'note',
                       'notify_to_add_document', 'notify_email')
        }),
    )
    search_fields = ('status', 'code', 'email', 'name')
    list_display = ('status', 'code', 'name', 'note', 'email', 'get_default_groups',
                    'get_default_users', 'get_edit_icon', 'get_delete_icon')
    list_display_csv = ('status', 'code', 'name', 'email')
    #list_display_links = ['name']
    detail_display = ('status', 'code', 'name', 'email')
    default_group_by = 'status'
    save_as = True
    save_on_top = True
    actions = None #['set_active', 'set_disactive']

    def get_default_groups(self, obj):
        return ','.join([ x.name for x in obj.get_default_groups() ])
    get_default_groups.short_description = _('Default Groups')

    def get_default_users(self, obj):
        return ','.join([ x.user.username for x in obj.get_default_users() ])
    get_default_users.short_description = _('Default Users')


class OtherCategoryAdmin(ExtendibleModelAdmin):
    class Media:
        css = {"all": ("document/css/admin_document.css",)}

    list_per_page = 30
    fieldsets = (
        (_('General Information'), {
            'fields': ('status', 'code', 'name', 'email', 'note',
                       'notify_to_add_document', 'notify_email')
        }),
    )
    search_fields = ('status', 'code', 'email', 'name')
    list_display = ('status', 'code', 'name', 'note', 'email', 'get_edit_icon', 'get_delete_icon')
    list_display_csv = ('status', 'code', 'name', 'email')
    #list_display_links = ['name']
    detail_display = ('status', 'code', 'name', 'email')
    default_group_by = 'status'
    save_as = True
    save_on_top = True
    actions = None #['set_active', 'set_disactive']


class DefaultAdmin(ExtendibleModelAdmin):
    class Media:
        css = {"all": ("document/css/admin_document.css",)}

    list_per_page = 30
    fieldsets = (
        (_('User Info'), {
            'fields': ('creator', 'date_create','last_modifier', 'date_last_modify',),
            'classes': ('collapse',)
        }),
        (_('General Information'), {
            'fields': ('category', 'users', 'groups')
        }),
    )
    list_display = ('category', 'get_string_users', 'get_string_groups')
    list_display_csv = ('category', 'get_users', 'get_groups')
    list_display_links = ['category']
    ordering = ['category']
    save_as = True
    save_on_top = True

admin.site.register(Settings, SettingsAdmin)
admin.site.register(Tipology)
admin.site.register(Category, CategoryAdmin)
admin.site.register(OtherCategory, OtherCategoryAdmin)
admin.site.register(Document, DocumentAdmin)
admin.site.register(DocumentProject, DocumentProjectAdmin)
admin.site.register(Default, DefaultAdmin)
admin.site.register(DocumentUser, DocumentUserAdmin)

########################################
# AUTOCOMPLETE REGISTER                #
########################################
def get_autocomplete_document_organization(request, query):
    results = []
    try:
        org_exact = Organization.objects.get(code=query)
        results.append([org_exact.id, "%s - %s" % (org_exact.code, org_exact.name)])
    except:
        pass

    q1 = Q(name__istartswith=query)
    q2 = Q(code__istartswith=query)
    for other_org in Organization.objects.exclude(code=query).filter(status=1).filter(q1|q2)[:10]:
        results.append([other_org.id, "%s - %s" % (other_org.code, other_org.name)])

    for other_org in Organization.objects.exclude(code=query).exclude(q1|q1).filter(name__icontains=query)[:10]:
        results.append([other_org.id, "%s - %s" % (other_org.code, other_org.name)])

    return results

def label_custom_code(obj):
    return "%s - %s" % (obj.code, obj.name)

autocomplete.register('document_organization', model = Organization, views = get_autocomplete_document_organization)
