# -*- coding: utf-8 -*-
from datetime import datetime
import os.path

from django.db import models
from django.contrib.auth.models import User, Group, Permission
from django.contrib.contenttypes.models import ContentType
from django.contrib.sites.models import Site

from django.utils.translation import ugettext_lazy as _
from django.utils.importlib import import_module
from django.utils.formats import date_format
from django.conf import settings as ls
from django.db.models import Q
from django.template.defaultfilters import slugify
from django.core.exceptions import PermissionDenied
from django.core.mail import EmailMessage, EmailMultiAlternatives

from jumbo.db.models import DateModel, UserModel, StatusModel, OrderedModel
from jumbo.db.models import Settings as JSettings
from jumbo.apps.project.models import Project
from jumbo.middleware.thread_local import get_current_user, get_request
from jumbo.utils.functions import get_settings

from organization.models import Organization, Contact


############################
# DOC USER                 #
############################
class DocumentUser(models.Model):
    user = models.OneToOneField (
        User,
        db_index = True,
        related_name = 'document_user',
        verbose_name = _("User")
    )
    enabled = models.BooleanField (
        db_index = True,
        verbose_name = _("Enabled")
    )
    class Meta:
        verbose_name = _("Document User")
        verbose_name_plural = _("Document Users")
        ordering = (
            'user__contact__organization', 'user__contact__last_name',
            'user__contact__first_name', 'user__username'
        )
        permissions = (
            ("view_documentuser",                       "Can view documentuser"),
            ("list_all_documentusers",                  "Can list all documentusers"),
        )

    def __unicode__(self):
        try:
            return u"%s - %s" % (self.user.contact.organization, self.user.contact)
        except:
            return u"%s" % self.user


############################
# CAT PERMISSIONS          #
############################
class CategoryPermission(Permission):
    model_id = models.PositiveIntegerField (
        db_index = True,
        verbose_name = _("Model ID")
    )


############################
# PROGETTO                 #
############################
class DocumentProject(Project):
    managers = models.ManyToManyField (
        DocumentUser,
        db_index = True,
        related_name = 'managed_documentprojects',
        verbose_name = _("Managers")
    )
    staff = models.ManyToManyField (
        DocumentUser,
        db_index = True,
        related_name = 'assigned_documentprojects',
        verbose_name = _("Staff")
    )

    class Meta:
        verbose_name = _("Document Project")
        verbose_name_plural = _("Document Projects")
        permissions = (
            ("view_documentproject",            "Can view document project"),
            ("list_all_documentprojects",       "Can list all document projects"),
        )
        ordering = ['name']

    def __unicode__(self):
        return unicode(self.name)

    def get_string_managers(self):
        if self.managers:
            managers = ['%s %s' % (u.last_name, u.first_name) for u in self.managers.all()]
            return ', '.join(managers)
        return ''
    get_string_managers.short_description = "managers"
    get_string_managers.admin_order_field = "managers"

    def get_string_staff(self):
        if self.staff:
            staff = ['%s %s' % (u.last_name, u.first_name) for u in self.staff.all()]
            return ', '.join(staff)
        return ''
    get_string_staff.short_description = "staff"
    get_string_staff.admin_order_field = "staff"

#############################
# CATEGORIA                 #
#############################
class BaseCategory(DateModel, UserModel, OrderedModel, StatusModel):
    ## Campi
    code = models.CharField(
        unique = True,
        max_length = 15,
        db_index = True,
        verbose_name = _('Code'),
    )
    name = models.CharField(
        unique = True,
        max_length = 30,
        db_index = True,
        verbose_name = _('Name'),
    )
    ## email utilizzata per il supporto Scan2Email
    email = models.EmailField(
        unique = True,
        verbose_name = _('Email'),
        help_text = _('Used for Scan2Email')
    )
    note = models.TextField(
        blank = True, null = True,
        verbose_name = _("Note"),
    )
    notify_to_add_document = models.BooleanField(
        default = False,
        verbose_name = _("Notify To Add Document"),
    )
    notify_email = models.TextField(
        blank = True, null = True,
        verbose_name = _("Notify Email"),
    )

    class Meta:
        abstract = True


    def __unicode__(self):
        return u"%s" % self.name

    def get_default_groups(self):
        defaults = self.document_defaults.all()
        if defaults:
            return defaults[0].groups.all()
        return []


    def get_default_users(self):
        defaults = self.document_defaults.all()
        if defaults:
            return defaults[0].users.all()
        return []


    def add_permissions(self, instance, content_type):
        if get_settings('document.settings', 'CREATE_DEFAULT_PERMS'):
            for perm in get_settings('document.settings', 'DEFAULT_PERMS'):
                permission = CategoryPermission()
                permission.name = perm[1] % instance.name
                permission.codename = perm[0] % slugify(instance.name)
                permission.content_type = content_type
                permission.model_id = instance.id
                permission.save()


    def change_permissions(self, instance, old_name):
        if get_settings('document.settings', 'CREATE_DEFAULT_PERMS'):
            if old_name != instance.name:
                for perm in get_settings('document.settings', 'DEFAULT_PERMS'):
                    permission = CategoryPermission.objects.get(codename = perm[0] % slugify(old_name))
                    permission.name = perm[1] % instance.name
                    permission.codename = perm[0] % slugify(instance.name)
                    permission.save()


class Category(BaseCategory):
    class Meta:
        verbose_name = _("Category")
        verbose_name_plural = _("Categories")
        ordering = ['name',]
        permissions = (
            ("view_category",             "Can view document category"),
            ("list_all_categories",       "Can list all document categories"),
        )

    def save(self, *args, **kwargs):
        if not self.email:
            name = self.name.replace(' ', '_').lower()
            self.email = "%s@%s" % (name, get_settings('document.settings', 'DOCUMENT_DOMAIN_NAME'))

        if not self.id:
            content_type = ContentType.objects.get_for_model(self)
            super(Category, self).save(*args, **kwargs)
            self.add_permissions(self, content_type)
        else:
            old_name = Category.objects.get(pk = self.pk).name
            super(Category, self).save(*args, **kwargs)
            self.change_permissions(self, old_name)


    def delete(self, *args, **kwargs):
        content_type = ContentType.objects.get_for_model(self)
        for perm in get_settings('document.settings', 'DEFAULT_PERMS'):
            f = CategoryPermission.objects.get(content_type = content_type, codename = perm[0] % slugify(self.name)).delete()
        super(Category, self).delete(*args, **kwargs)


class OtherCategory(BaseCategory):
    class Meta:
        verbose_name = _("Other Category")
        verbose_name_plural = _("Other Categories")
        ordering = ['name',]
        permissions = (
            ("view_other_category",             "Can view document other category"),
            ("list_all_other_categories",       "Can list all document other categories"),
        )


    def save(self, *args, **kwargs):
        if not self.email:
            name = self.name.replace(' ', '_').lower()
            self.email = "%s@%s" % (name, get_settings('document.settings', 'DOCUMENT_DOMAIN_NAME'))

        if not self.id:
            content_type = ContentType.objects.get_for_model(self)
            super(OtherCategory, self).save(*args, **kwargs)
            self.add_permissions(self, content_type)
        else:
            old_name = OtherCategory.objects.get(pk = self.pk).name
            super(OtherCategory, self).save(*args, **kwargs)
            self.change_permissions(self, old_name)


    def delete(self, *args, **kwargs):
        content_type = ContentType.objects.get_for_model(self)
        for perm in get_settings('document.settings', 'DEFAULT_PERMS'):
            f = CategoryPermission.objects.get(content_type = content_type, codename = perm[0] % slugify(self.name)).delete()
        super(OtherCategory, self).delete(*args, **kwargs)


#############################
# DEFAULT                   #
#############################
class Default(DateModel, UserModel):
    category = models.ForeignKey(
        Category,
        unique = True,
        db_index = True,
        related_name = "document_defaults",
        verbose_name = _("Category"),
    )
    users = models.ManyToManyField(
        DocumentUser,
        db_index = True,
        blank = True, null = True,
        related_name = "document_defaults",
        verbose_name = _("Users"),
    )
    groups = models.ManyToManyField(
        Group,
        db_index = True,
        blank = True, null = True,
        related_name = "document_defaults",
        verbose_name = _("Groups"),
    )
    class Meta:
        verbose_name = _("Default")
        verbose_name_plural = _("Defaults")

    def __unicode__(self):
        return u"%s" % (self.category.name)

    def get_string_users(self):
        if self.users:
            users = ['%s %s' % (u.user.last_name, u.user.first_name) for u in self.users.all()]
            return ', '.join(users)
        return ''
    get_string_users.short_description = "users"
    get_string_users.admin_order_field = "users"

    def get_string_groups(self):
        if self.groups:
            groups = [g.name for g in self.groups.all()]
            return ', '.join(groups)
        return ''
    get_string_groups.short_description = "groups"
    get_string_groups.admin_order_field = "groups"

def get_document_upload_to(instance, filename):
    if instance.category:
        return "documents/%s/%s/%s" % (instance.category.name, instance.category.name[0].upper(), filename)
    else:
        return "documents/no_category"

def get_document_etichetta_upload_to(instance, filename):
    if instance.category:
        return "documents/%s/%s/etichette/%s" % (instance.category.name, instance.category.name[0].upper(), filename)
    else:
        return "documents/no_category/etichette"



class Tipology(OrderedModel, StatusModel):
    name = models.CharField(
        db_index = True,
        max_length = 50,
        verbose_name = _('Tipology'),
    )

    class Meta:
        verbose_name = _("Tipology")
        verbose_name_plural = _("Tipologies")
        ordering = ['name',]
        permissions = (
            ("view_tipology",             "Can view document tipology"),
            ("list_all_tipologies",       "Can list all document tipologies"),
        )

    def __unicode__(self):
        return u"%s" % self.name

#############################
# DOCUMENTO                 #
#############################
class DocumentProtocolManager(models.Manager):
    def get_query_set(self):
        return super(DocumentProtocolManager, self).get_query_set()

class DocumentManager(models.Manager):
    def get_query_set(self):
        user = get_current_user()
        groups = user.groups.all()
        qs = super(DocumentManager, self).get_query_set()

        ## se può vedere tutti i documenti ritorno il QS non filtrato
        #if user.has_perm("document.list_all_documents"):
        #    return qs

        # Il Q iniziale e' quello di filtrare per tutti i documenti pubblici che
        # sicuramente saranno visibili da tutti
        q = Q(is_public=True)

        ## se può vedere i documenti dei propri GRUPPI
        if user.has_perm("document.list_group_documents") and groups:
            q1 = Q(is_public=False, notify_group__in=groups)
            q2 = Q(is_public=False, assigned_to__user=user)
            q = q | q1 | q2

        ## se può vedere i documenti della propria AZIENDA
        try:
            u_contact = user.contact
            u_organization = user.contact.organization
            if (
                user.has_perm("document.list_organization_documents")
                and u_contact and u_organization
            ):
                q1 = Q(is_public=False, organization=u_organization)
                q2 = Q(is_public=False, assigned_to__user=user)
                q = q | q1 | q2
        except:
            u_contact = None
            u_organization = None

        ### se può vedere i documenti PROPRI
        if user.has_perm("document.list_own_documents"):
            q1 = Q(is_public=False, assigned_to__user=user)
            q = q | q1

        ### se l'utente è autenticato
        if user.is_authenticated():
            ### list_lperms è la lista di permessi "LIST"
            ### creati per la visualizzazione dei documenti
            list_lperms = Permission.objects.filter(
                categorypermission__model_id__isnull=False,
                codename__startswith='list'
            )
            
            for perm in list_lperms:
            
                # perm = document | Category | Can list all Contratti Clienti documents
                # perm.codename = list_all_contratti-clienti_documents
                # ptype = all (all | group | organization | own)
                # pcat = id modello
                # pgroups = gruppi che hanno il permsso 'perm'
                
                perm_split = perm.codename.split("_")
                ## ptype: tipo di visualizzazione - "all" "group" "organization" "own"
                ptype = perm_split[1]

                ## recupero la Categoria Documento alla quale il permesso si riferisce
                pcat = perm.categorypermission.model_id

                ## recupero i Gruppi che hanno il permesso "perm"
                pgroups = perm.group_set.all()
                ## recupero solo i Gruppi "pgroups" di cui l'utente fa parte
                intersect_group = set(groups).intersection(set(pgroups))

                if user.has_perm("document.%s" % perm.codename):
                    other_cat_ct = ContentType.objects.get_for_model(OtherCategory)
                    if ptype == "all":
                        if other_cat_ct.id == perm.content_type.id:
                            q1 = Q(other_category=pcat)
                        else:
                            q1 = Q(category=pcat)
                        q = q | q1
                    elif ptype == "group" and pgroups and groups:
                        if other_cat_ct.id == perm.content_type.id:
                            q1 = Q(notify_group__in=intersect_group, other_category=pcat)
                        else:
                            q1 = Q(notify_group__in=intersect_group, category=pcat)
                        q = q | q1
                    elif ptype == "organization" and u_organization:
                        if other_cat_ct.id == perm.content_type.id:
                            q1 = Q(organization=u_organization, other_category=pcat)
                        else:
                            q1 = Q(organization=u_organization, category=pcat)
                        q = q | q1
                    elif ptype == "own":
                        if other_cat_ct.id == perm.content_type.id:
                            q1 = Q(assigned_to__user=user, other_category=pcat)
                        else:
                            q1 = Q(assigned_to__user=user, category=pcat)

                        q = q | q1
                        
            q1 = Q(is_public=False, downloadable_by__user=user)
            q = q | q1

        return qs.filter(q).distinct()

class Document(DateModel, UserModel, OrderedModel):
    # Foreign Key
    projects = models.ManyToManyField(
        DocumentProject,
        null = True, blank = True,
        db_index = True,
        related_name = "documents",
        verbose_name = _("Projects"),
    )
    notify_group = models.ManyToManyField (
        Group,
        blank = True, null = True,
        db_index = True,
        related_name = "documents",
        verbose_name = _("notify_group"),
    )
    organization = models.ForeignKey(
        Organization,
        null = True, blank = True,
        db_index = True,
        related_name = "documents",
        verbose_name = _("Organization"),
    )
    person = models.ForeignKey(
        Contact,
        null = True, blank = True,
        related_name = "documents",
        verbose_name = _("Person"),
    )
    assigned_by = models.ForeignKey (
        DocumentUser,
        blank = True, null = True,
        related_name = "documents_assigned_by_me",
        verbose_name = _("Assigned by"),
    )
    assigned_to = models.ForeignKey (
        DocumentUser,
        blank = True, null = True,
        related_name = "documents_assigned_to_me",
        verbose_name = _("Assigned to"),
    )
    category = models.ForeignKey(
        Category,
        db_index = True,
        related_name = "documents",
        verbose_name = _("Category"),
    )
    tipology = models.ForeignKey(
        Tipology,
        db_index = True,
        blank = True, null = True,
        related_name = "documents",
        verbose_name = _("Tipology"),
    )
    other_category = models.ManyToManyField(
        OtherCategory,
        db_index = True,
        blank = True, null = True,
        related_name = "documents",
        verbose_name = _("Category"),
    )
    # Campi Modello
    status = models.CharField(
        max_length = 2,
        choices = get_settings('document.settings', 'STATUS_DOC'),
        db_index = True,
        verbose_name = _("Status"),
    )
    date_doc = models.DateTimeField(
        null = True, blank = True,
        db_index = True,
        verbose_name = _("Document Date and Time"),
        ## campo obbligatorio quando in stato "COMPLETATO"
    )
    number = models.CharField(
        null = True, blank = True,
        max_length = 20,
        db_index = True,
        verbose_name = _("Document Number"),
        db_column = "dnumber"
        ## obbligatorio per raccomandate
    )
    working = models.BooleanField(
        default = False,
        db_index = True,
        verbose_name = _("Working")
    )
    reception_date = models.DateTimeField(
        db_index = True,
        verbose_name = _("Reception Date"),
    )
    protocol_number = models.IntegerField(
        null = True, blank = True,
        db_index = True,
        verbose_name = _("Protocol Number"),
    )
    title = models.CharField(
        max_length = 100,
        db_index = True,
        verbose_name = _("Title"),
    )
    keywords = models.CharField(
        null = True, blank = True,
        max_length = 200,
        db_index = True,
        verbose_name = _("Keywords"),
    )
    pages = models.PositiveSmallIntegerField(
        null = True, blank = True,
        max_length = 3,
        verbose_name = _("Pag."),
    )
    support = models.CharField(
        max_length = 1,
        choices = get_settings('document.settings', 'SUPPORTO_DOC'),
        db_index = True,
        verbose_name = _("Support"),
    )
    file = models.FileField (
        upload_to = get_document_upload_to,
        verbose_name = _("File PDF"),
    )
    etichetta = models.FileField (
        upload_to = get_document_etichetta_upload_to,
        verbose_name = _("Etichetta PDF"),
        blank = True, null = True
    )
    note = models.TextField (
        null = True, blank = True,
        verbose_name = _("Note"),
    )
    is_public = models.BooleanField(
        default = False
    )
    direction = models.CharField(
        max_length = 1,
        null = True, blank = True,
        choices = get_settings('document.settings', 'DIRECTION_DOC'),
        db_index = True,
        verbose_name = _("Direction")
    )
    downloadable_by = models.ManyToManyField(
        DocumentUser,
        db_index = True,
        blank = True, null = True,
        related_name = "documents",
        verbose_name = _("Downloadable By"),
    )
    objects = DocumentManager()
    for_protocol = DocumentProtocolManager()

    class Meta:
        verbose_name = _("Document")
        verbose_name_plural = _("Documents")
        permissions = (
            ("view_document",                       "Can view document"),
            ("list_all_documents",                  "Can list all documents"),
            ("list_group_documents",                "Can list group documents"),
            ("list_organization_documents",         "Can list organization documents"),
            ("list_own_documents",                  "Can list own documents"),
            ("add_all_documents",                   "Can add all documents"),
            ("change_all_documents",                "Can change all documents"),
            ("delete_all_documents",                "Can delete all documents"),
            ("download_all_documents",              "Can download all documents"),
        )


    def send_mail_add_document_to_category(self, category, change):
        subject = get_settings('document.settings', "DOCUMENT_MAIL_CATEGORY_SUBJECT")
        from_mail = get_settings('document.settings', "DOCUMENT_MAIL_CATEGORY_FROM")

        if change:
            user = self.last_modifier
            action = _("modified")
        else:
            user = self.creator
            action = _("added")

        all_mails = []
        emails = category.notify_email.split('\r\n')
        for e in emails:
            for email in e.split(','):
                all_mails.append(email.strip())

        subject_mail = subject % {'user':user, 'action':action,
                                  'document':self.title, 'category':category.name}

        site = Site.objects.get_current()

        BODY_MSG = 'Click on this link to download the file: http://%s%s' % (site, self.file.url)
        BODY_HTML_MSG = 'Click on this link to download the file:<br/><a href="http://%s%s">http://%s%s</a>' % (site, self.file.url, site, self.file.url)

        email = EmailMultiAlternatives(subject_mail,
                                     BODY_MSG,
                                     from_mail,
                                     all_mails)
        email.attach_alternative(BODY_HTML_MSG, "text/html")
        email.send()

        #email = EmailMessage(subject_mail, , from_mail, all_mails)
        #email.attach_file(self.file.path)
        #email.send()


    def get_string_projects(self):
        if self.projects:
            projects = [p.name for p in self.projects.all()]
            return ', '.join(projects)
        return ''

    def save(self, scan_to_mail=False):
        from jumbo.middleware.thread_local import get_current_user
        user = get_current_user()
        groups = user.groups.all()
        can_be_save = False

        if user:
            self.utente = user

        # Se il salvataggio viene chiamato dalla procedure dello scan to mail
        # salvo senza controllare i permessi
        if scan_to_mail:
            can_be_save = True

        ## INSERIMENTO nuuovo documento
        if not self.id:
            if self.can_be_added():
                can_be_save = True

        ## MODIFICA documento esistente
        else:
            if self.can_be_modified():
                can_be_save = True

        if can_be_save:
            if self.status == 'CM' and not self.protocol_number:
                self.protocol_number = self.get_new_protocol_number()
            return super(Document, self).save()

        raise PermissionDenied


    def delete(self, *args, **kws):
        if self.can_be_delete():
            return super(Document, self).delete(*args, **kws)
        raise PermissionDenied


    def is_busy(self):
        ## un documento è impegnato da altri quando 'user' è diverso dal 'current_user'
        ## e self.documento_in_lavorazione è True
        from jumbo.middleware.thread_local import get_current_user
        user = get_current_user()
        list_perms = user.get_all_permissions()

        ## controlla se è STAFF
        if user.is_staff:
            return False

        if self.last_modifier != user and self.working == True:
            return True

        return False


    def can_be_added(self):
        from jumbo.middleware.thread_local import get_current_user
        user = get_current_user()

        ## controlla permessi di modifica "STAFF"
        if user.has_perm("document.add_document"):
            return True

        ## controlla se inserimento da WEB
        if user.has_perm("document.can_insert_by_web"):
            return True

        return False


    def can_be_modified(self):
        from jumbo.middleware.thread_local import get_current_user
        user = get_current_user()

        # Deve avere per forza change_document xke altrimenti non gli viene presentato
        # il form di change di Django
        if not user.has_perm("document.change_document"):
            return False

        ## controlla se il documento non è già impegnato da un altro
        if self.is_busy():
            return False

        return self.generic_check_permissions(user, "change")


    def can_be_delete(self):
        from jumbo.middleware.thread_local import get_current_user
        user = get_current_user()

        # Deve avere per forza delete_document xke altrimenti Django non gli
        # permette di cancellare
        if not user.has_perm("document.delete_document"):
            return False

        return self.generic_check_permissions(user, "delete")

    def can_be_downloaded(self):
        from jumbo.middleware.thread_local import get_current_user
        user = get_current_user()

        user_can_download = [ docuser.user.username for docuser in self.downloadable_by.all() ]

        # Se e' stato settato qualche utente nella multiselect allora faccio
        # scaricare il documento solo a loro. Se vuota il documento lo possono
        # scaricare tutti
        if not user_can_download:
            return True

        if user.username in user_can_download or self.creator.username == user.username:
            return True

        return False

    def generic_check_permissions(self, user, action):
        # QUERYSET OTHER CATEGORIE
        qs_other_cat = self.other_category.all()

        #########################
        # PERMESSI GENERALI
        #########################

        ## controlla permessi di modifica "ALL" o "STAFF"
        if user.has_perm("document.%s_all_documents" % action):
            return True

        # PER LE CATEGORIE
        cat_perm = "document.%s_all_%s_documents" % (action, slugify(self.category.name))
        if user.has_perm(cat_perm):
            return True

        # PER LE ALTRE CATEGORIE
        for c in qs_other_cat:
            other_cat_perm = "document.%s_all_%s_documents" % (action, slugify(c.name))
            if user.has_perm(other_cat_perm):
                return True

        #########################
        # PERMESSI DI OWN E GROUP
        #########################
        user_groups = user.groups.all()
        notify_groups = self.notify_group.all()

        # PER LE CATEGORIE
        # -----------------------
        # OWN
        cat_own_perm = "document.%s_own_%s_documents" % (action, slugify(self.category.name))
        if user.has_perm(cat_own_perm) and self.assigned_to and self.assigned_to.user == user:
            return True

        # GROUP
        cat_group_perm = "document.%s_group_%s_documents" % (action, slugify(self.category.name))
        if user.has_perm(cat_group_perm):
            for g in notify_groups:
                if g in user_groups:
                    return True

        # PER LE ALTRE CATEGORIE
        # -----------------------
        for c in qs_other_cat:
            # OWN
            other_cat_own_perm = "document.%s_own_%s_documents" % (action, slugify(c.name))
            if user.has_perm(other_cat_own_perm) and self.assigned_to and self.assigned_to.user == user:
                return True

            # GROUP
            other_cat_group_perm = "document.%s_group_%s_documents" % (action, slugify(c.name))
            if user.has_perm(other_cat_group_perm):
                for g in notify_groups:
                    if g in user_groups:
                        return True

        return False


    def get_new_protocol_number(self):
        # Dato che la funzione che torna il numero del protocollo puo' cambiare
        # da cliente a cliente, la metto nei settings del progetto in modo che possa
        # essere personalizzata in maniera semplice.
        # Se non viene ridefinita di default viene presa quella in document.utils.get_new_protocol_number
        # Per ridefinirla basta aggiugnere nei settings.py del proprio progetto la costante:
        #
        # PROTOCOL_NUMBER_FUNCTION = 'modulo.funzione'
        #
        PROTOCOL_NUMBER_FUNCTION = get_settings('document.settings', 'PROTOCOL_NUMBER_FUNCTION')
        module_name = import_module('.'.join(PROTOCOL_NUMBER_FUNCTION.split('.')[:-1]))
        function_name = PROTOCOL_NUMBER_FUNCTION.split('.')[-1]
        return getattr(module_name, function_name)(self)


    def set_num_doc(self, number):
        self.number = number
        self.save()

    def get_reception_date(self):
        if self.reception_date:
            return date_format(self.reception_date)
        return ""

    def get_other_category(self):
        g = [ x.name for x in self.other_category.all() ]
        return ', '.join(g)

    def __unicode__(self):
        category = ''
        if self.category:
            category = self.category
        return u"%s/%s %s" % (self.protocol_number, self.date_doc, category)

class Link(DateModel, UserModel, OrderedModel):
    number = models.IntegerField(
        db_index = True,
        verbose_name = _("Link Number"),
        db_column = "lnumber"
    )
    document = models.ForeignKey(
        Document,
        db_index = True,
        related_name = "links",
        verbose_name = _("Document"),
    )

    class Meta:
        verbose_name = _("Document Link")
        verbose_name_plural = _("Document Links")
        unique_together = (('number', 'document'),)

    def documents_linked(self):
        return Document.objects.filter(links__number=1).distinct()

    def __unicode__(self):
        return u"%s" % self.number

    def save(self):
        if not self.number:
            try:
                last_numb = Link.objects.all().order_by('-number')[0].number
            except:
                last_numb = 0
            self.number = last_numb + 1
        super(Link, self).save()

class Settings(JSettings):
    pass



